
% All programs in this collection are free software: 
% you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright 2007-2008 Alexander Zien.
% Max Planck Intitute for Biological Cybernetics, Tbingen, Germany.
% Friedrich Miescher Lab of the Max Planck Society, Tbingen, Germany.


clear;
addpath( '/fml/ag-raetsch/home/ong/MatlabCode/cplexint100/' );


% === const
datasetName = 'psortPos';
kernelNum = 1;
fracTst = 0.2;
C = 0.1;
epsCond = 1e-12;


% === init
labelFileName = sprintf( '%s/label_%s.mat', datasetName, datasetName );
klistFileName = sprintf( '%s/klist_%s.mat', datasetName, datasetName );
permFileName = sprintf( '%s/perms,%s.mat', datasetName, datasetName );
load( labelFileName, 'y' );
load( klistFileName, 'klist' );
load( permFileName, 'perms' );
% --- init
classes = unique( y )';
nofClasses = length( classes );
[ nofPerms, n ] = size( perms );
ASSERT( size(y) == [n,1] );
nofTst = round( n * fracTst );
nofTrn = n - nofTst;


% === kernel
fprintf( '%2d  %s', kernelNum, klist{kernelNum} );
kernelFileName = sprintf( '%s/%s', datasetName, klist{kernelNum} );
load( kernelFileName, 'K' );
ASSERT( size(K) == [n,n] );
s = mean(diag(K)) - mean(K(:));
K = (1/s) * K;
e = eig( K );
cond = min(e) / max(e);
if( cond < epsCond )
  K = K + (epsCond-cond) * max(e) * speye(n,n);
end;


% === param
param = [];
param.C = C;
optParam = [];
optParam.optimizer = 'cplexint';
optParam = MP_getLicence( optParam, 'permanentLicence' );
param.optParam = optParam;


% === mcSVM
allErrs = [];
for( p = 1:nofPerms )
  perm = perms(p,:);
  idxTst = perms( p, 1:nofTst );
  idxTrn = perms( p, (nofTst+1):end );
  assert( length(unique(y(idxTrn))) == nofClasses );
  [ svm, param ] = mcSvmTrainCompr( param, [], K(idxTrn,idxTrn), y(idxTrn) );
  [ preds, outs ] = mcSvmPred( param, svm, K(:,idxTrn) );
  errVec = ( preds ~= y );
  errTrn = mean( errVec(idxTrn) );
  errTst = mean( errVec(idxTst) );
  allErrs = [ allErrs ; errTst , errTrn ];
  meanErrs = mean( allErrs, 1 );
  fprintf( 'mean error: %5.2f%% (train %5.2f%%)\n', 100*meanErrs );
end;


% === clean up
optParam = MP_releaseLicence( optParam, 'permanentLicence' );


