
% All programs in this collection are free software: 
% you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright 2007-2008 Alexander Zien.
% Max Planck Intitute for Biological Cybernetics, Tbingen, Germany.
% Friedrich Miescher Lab of the Max Planck Society, Tbingen, Germany.


clear;
addpath( '/fml/ag-raetsch/share/software/matlab_tools/cplex9/' );


% === const (play with these!)
datasetName = 'psortPos';
fracTst = 0.2;
C = 1;


% === init
labelFileName = sprintf( '%s/label_%s.mat', datasetName, datasetName );
klistFileName = sprintf( '%s/klist_%s.mat', datasetName, datasetName );
permFileName = sprintf( '%s/perms,%s.mat', datasetName, datasetName );
load( labelFileName, 'y' );
load( klistFileName, 'klist' );
load( permFileName, 'perms' );
% --- init
classes = unique( y )';
nofClasses = length( classes );
[ nofPerms, n ] = size( perms );
ASSERT( size(y) == [n,1] );
nofTst = round( n * fracTst );
nofTrn = n - nofTst;


% === kernels
nofKernels = length( klist );
Kxs = repmat( nan, [n,n,nofKernels] );
for( k = 1:nofKernels )
  kernelNum = k;
  fprintf( '%2d  %s', kernelNum, klist{kernelNum} );
  kernelFileName = sprintf( '%s/%s', datasetName, klist{kernelNum} );
  load( kernelFileName, 'K' );
  ASSERT( size(K) == [n,n] );
  s = mean(diag(K)) - mean(K(:));
  K = (1/s) * K;
  Kxs(:,:,k) = K;
  scales(k) = s;
  fprintf( '  %.2f\n', s );
end;
clear K;


% === param
% --- optimization
optParam = [];
optParam.optimizer = 'cplexint';
optParam = MP_getLicence( optParam, 'permanentLicence' );
% --- mcSVM
baseParam = [];
baseParam.C = C;
baseParam.optParam = optParam;
baseParam.Ky = eye( nofClasses );
% --- silp
mklParam = [];
mklParam.optParam = optParam;
mklParam.verbosity = 0;
mklParam.normKernels = 'var';
mklParam.baseLearner = @mcSvmTrainCompr;
mklParam.sumOfAlphas = @mcSvmSumOfAlphas;
mklParam.evalRegularizer = @mcSvmRegularizer;


% === MCMKL
allErrs = [];
allBetas = [];
allSvms = cell( 1, nofPerms );
for( p = 1:nofPerms )
  perm = perms(p,:);
  idxTst = perms( p, 1:nofTst );
  idxTrn = perms( p, (nofTst+1):end );
  assert( length(unique(y(idxTrn))) == nofClasses );
  [ betas, svm ] = mklSilp( baseParam, mklParam, Kxs, y, idxTrn' );
  allBetas = [ allBetas , betas ];
  allSvms{p} = svm;
  % --- eval
  K = mixKernel( Kxs, betas );
  [ preds, outs ] = mcSvmPred( baseParam, svm, K(:,idxTrn) );
  errVec = ( preds ~= y );
  errTrn = mean( errVec(idxTrn) );
  errTst = mean( errVec(idxTst) );
  allErrs = [ allErrs ; errTst , errTrn ];
  meanErrs = mean( allErrs, 1 );
  fprintf( 'mean error: %5.2f%% (train %5.2f%%)\n', 100*meanErrs );
end;


% === done
fileName = sprintf( 'res,%s,C=%.2f.mat', datasetName, C );
save( fileName, 'allBetas', 'allSvms', 'allErrs' );
optParam = MP_releaseLicence( optParam, 'permanentLicence' );


